@@ -3,15 +3,16 @@
     reason = "Prefer System trait methods over std methods in ty crates"
 )]
 mod all_symbols;
+mod code_action;
 mod completion;
 mod doc_highlights;
 mod docstring;
 mod document_symbols;
 mod find_node;
+mod find_references;
 mod goto;
 mod goto_declaration;
 mod goto_definition;
-mod goto_references;
 mod goto_type_definition;
 mod hover;
 mod importer;
@@ -27,13 +28,16 @@ mod symbols;
 mod workspace_symbols;
 
 pub use all_symbols::{AllSymbolInfo, all_symbols};
+pub use code_action::{QuickFix, code_actions};
 pub use completion::{Completion, CompletionKind, CompletionSettings, completion};
 pub use doc_highlights::document_highlights;
 pub use document_symbols::document_symbols;
+pub use find_references::find_references;
 pub use goto::{goto_declaration, goto_definition, goto_type_definition};
-pub use goto_references::goto_references;
 pub use hover::hover;
-pub use inlay_hints::{InlayHintKind, InlayHintLabel, InlayHintSettings, inlay_hints};
+pub use inlay_hints::{
+    InlayHintKind, InlayHintLabel, InlayHintSettings, InlayHintTextEdit, inlay_hints,
+};
 pub use markup::MarkupKind;
 pub use references::ReferencesMode;
 pub use rename::{can_rename, rename};
@@ -226,6 +230,11 @@ impl NavigationTargets {
     fn is_empty(&self) -> bool {
         self.0.is_empty()
     }
+
+    #[cfg(test)]
+    fn len(&self) -> usize {
+        self.0.len()
+    }
 }
 
 impl IntoIterator for NavigationTargets {
