@@ -573,16 +573,16 @@ impl Workspace {
         // This is only for actions that are messy to compute at the time of the diagnostic.
         // For instance, suggesting imports requires finding symbols for the entire project,
         // which is dubious when you're in the middle of resolving symbols.
-        if let Some(range) = diagnostic.inner.range()
-            && let Some(fixes) = ty_ide::code_actions(
-                &self.db,
-                file_id.file,
-                range,
-                diagnostic.inner.id().as_str(),
-            )
-        {
-            for action in fixes {
-                actions.push(CodeAction {
+        if let Some(range) = diagnostic.inner.range() {
+            actions.extend(
+                ty_ide::code_actions(
+                    &self.db,
+                    file_id.file,
+                    range,
+                    diagnostic.inner.id().as_str(),
+                )
+                .into_iter()
+                .map(|action| CodeAction {
                     title: action.title,
                     preferred: action.preferred,
                     edits: action
@@ -590,8 +590,8 @@ impl Workspace {
                         .into_iter()
                         .map(|edit| edit_to_text_edit(self, file_id.file, &edit))
                         .collect(),
-                });
-            }
+                }),
+            );
         }
 
         if actions.is_empty() {
